//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*

#include "File.h"
#include "Configuration.h"
#include "OptionsDialog.h"
#include "defs.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
//#define debug(x)

OptionsDialog::OptionsDialog(PG_Widget* parent, PG_Rect rect)
    :PG_Window(parent, rect, _("Game settings"),PG_Window::MODAL)
{
    debug("OptionsDialog()");

    d_cb_smooth_scrolling = new PG_CheckButton(this, PG_Rect(10, 40, Width() - 20, 30), _("Smooth scrolling"),0);
    d_cb_show_next_player = new PG_CheckButton(this, PG_Rect(10, 70, Width() - 20, 30), _("Show next player popup"),0);

    /* Set initial state */
    if(Configuration::s_smoothScrolling)
        d_cb_smooth_scrolling->SetPressed();

    if(Configuration::s_showNextPlayer)
        d_cb_show_next_player->SetPressed();

    d_b_ok = new PG_Button(this, PG_Rect(5, Height() - 40, 0, 30), _("OK"));
    d_b_ok->SizeWidget(d_b_ok->GetTextWidth()+40,30,true);

    d_b_cancel = new PG_Button(this, PG_Rect(d_b_ok->GetTextWidth()+55, Height() - 40, 0, 30), _("Cancel"));
    d_b_cancel->SizeWidget(d_b_cancel->GetTextWidth()+40,30,true);

    d_b_help = new PG_Button(this, PG_Rect(d_b_ok->GetTextWidth()+55+d_b_cancel->GetTextWidth()+50, Height() - 40, 0, 30), _("Help"));
    d_b_help->SizeWidget(d_b_help->GetTextWidth()+40,30,true);

    d_b_ok->sigClick.connect(slot(*this, &OptionsDialog::okClicked));
    d_b_cancel->sigClick.connect(slot(*this, &OptionsDialog::cancelClicked));
    d_b_help->sigClick.connect(slot(*this, &OptionsDialog::helpClicked));

}

OptionsDialog::~OptionsDialog()
{
    delete d_b_ok;
    delete d_b_cancel;
    delete d_b_help;
    delete d_cb_show_next_player;
    delete d_cb_smooth_scrolling;
}

bool OptionsDialog::okClicked(PG_Button* btn)
{
    /* Transfer button state to configuration */
    Configuration::s_showNextPlayer = d_cb_show_next_player->GetPressed();
    Configuration::s_smoothScrolling = d_cb_smooth_scrolling->GetPressed();
    QuitModal();
    return true;
}
bool OptionsDialog::cancelClicked(PG_Button* btn)
{
    QuitModal();
    return true;
}
bool OptionsDialog::helpClicked(PG_Button* btn)
{
    debug("Not yet implemented");
    QuitModal();
    return true;
}

// End of file
