//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef RWINGAME_H
#define RWINGAME_H

#include <sigc++/sigc++.h>
#include "Reaction.h"

/** This reaction determines that the game has been "won" (whatever that means).
  * Usually (with some more patching), it will show the win dialog.
  *
  * To make it a bit more complex, you can supply a number with this reaction.
  * This number can be used later to signal _how_ the game was won (for
  * non-linear campaigns). However, currently this is only a vague idea.
  */

class RWinGame : public Reaction
{
    public:
        RWinGame(int status);
        RWinGame(XML_Helper* helper);
        ~RWinGame();

        //! Saves the reaction data
        bool save(XML_Helper* helper);

        //! Triggers the reaction
        bool trigger();

        
        //! Returns the status for this reaction
        int getStatus() const {return d_status;}

        //! Sets the status for this reaction
        void setStatus(int status) {d_status = status;}
        
        
        //! signal is connected outside of the scope of this class
        static SigC::Signal1<void, int> swinning;

    private:
        int d_status;
};

#endif //RWINGAME_H
